/*
 * This file originates from The Android Open Source Project version 4.4.2_r1
 * and contains some modifications which are (C) COPYRIGHT 2014 ARM Limited
 */

/*
 * This file is included from rs_mali_element.c, where PREFIX
 * and ADDR_SPACE are defined.
 */

#include "rs_core.rsh"
#include "rs_structs.h"
#include "rs_mali.h"

// ARM - exclude unused runtime functions
// ARM - Add macros to prefix function names depending on their address spaces
// ARM - Add macros to allow the address space to be declared

/**
* Element
*/
#if 0 // not used by mali GPU driver
extern uint32_t __attribute__((overloadable))
        rsElementGetSubElementCount(rs_element e) {
    Element_t *element = (Element_t *)e.p;
    if (element == NULL) {
        return 0;
    }
    return element->mHal.state.fieldsCount;
}

extern rs_element __attribute__((overloadable))
        rsElementGetSubElement(rs_element e, uint32_t index) {
    Element_t *element = (Element_t *)e.p;
    if (element == NULL || index >= element->mHal.state.fieldsCount) {
        rs_element nullElem = {0};
        return nullElem;
    }
    rs_element returnElem = {element->mHal.state.fields[index]};
    return returnElem;
}

extern uint32_t __attribute__((overloadable))
        rsElementGetSubElementNameLength(rs_element e, uint32_t index) {
    Element_t *element = (Element_t *)e.p;
    if (element == NULL || index >= element->mHal.state.fieldsCount) {
        return 0;
    }
    return element->mHal.state.fieldNameLengths[index];
}

extern uint32_t __attribute__((overloadable))
        rsElementGetSubElementName(rs_element e, uint32_t index, char *name, uint32_t nameLength) {
    Element_t *element = (Element_t *)e.p;
    if (element == NULL || index >= element->mHal.state.fieldsCount ||
        nameLength == 0 || name == 0) {
        return 0;
    }

    uint32_t numToCopy = element->mHal.state.fieldNameLengths[index];
    if (nameLength < numToCopy) {
        numToCopy = nameLength;
    }
    // Place the null terminator manually, in case of partial string
    numToCopy --;
    name[numToCopy] = '\0';
    const char *nameSource = element->mHal.state.fieldNames[index];
    for (uint32_t i = 0; i < numToCopy; i ++) {
        name[i] = nameSource[i];
    }
    return numToCopy;
}

extern uint32_t __attribute__((overloadable))
        rsElementGetSubElementArraySize(rs_element e, uint32_t index) {
    Element_t *element = (Element_t *)e.p;
    if (element == NULL || index >= element->mHal.state.fieldsCount) {
        return 0;
    }
    return element->mHal.state.fieldArraySizes[index];
}

extern uint32_t __attribute__((overloadable))
        rsElementGetSubElementOffsetBytes(rs_element e, uint32_t index) {
    Element_t *element = (Element_t *)e.p;
    if (element == NULL || index >= element->mHal.state.fieldsCount) {
        return 0;
    }
    return element->mHal.state.fieldOffsetBytes[index];
}
#endif // not used by mali GPU driver

extern uint32_t __attribute__((overloadable))
        PREFIX(ElementGetBytesSize)(rs_element e) {
    ADDR_SPACE mali_Element_t * element = (ADDR_SPACE mali_Element_t *)e.p;
    if (!element) {
        return 0;
    }
    return element->mHal.state.elementSizeBytes;
}

extern rs_data_type __attribute__((overloadable))
        PREFIX(ElementGetDataType)(rs_element e) {
    ADDR_SPACE mali_Element_t * element = (ADDR_SPACE mali_Element_t *)e.p;
    if (!element) {
        return RS_TYPE_INVALID;
    }
    return element->mHal.state.dataType;
}

extern rs_data_kind __attribute__((overloadable))
        PREFIX(ElementGetDataKind)(rs_element e) {
    ADDR_SPACE mali_Element_t * element = (ADDR_SPACE mali_Element_t *)e.p;
    if (!element) {
        return RS_KIND_INVALID;
    }
    return element->mHal.state.dataKind;
}

extern uint32_t __attribute__((overloadable))
        PREFIX(ElementGetVectorSize)(rs_element e) {
    ADDR_SPACE mali_Element_t * element = (ADDR_SPACE mali_Element_t *)e.p;
    if (!element) {
        return 0;
    }
    return element->mHal.state.vectorSize;
}
